#####################################################
# To analyze chr13 with fragment length 200 and bin
# size of 50 and collapse to 3 counts, type 
# perl preprocess_chip.pl chr13 200 50 3 dewey_infile.txt
# NOTE: FIX read length to be 28
#####################################################

#!/usr/bin/env perl;
use warnings;
use strict;
use FindBin;
use lib $FindBin::Bin;
$|=1;

my ($chr, $L, $binsize, $collapse, @filename) = @ARGV;

my @cutfile=();
my $len = scalar(@filename)-1;

for(my $i=0; $i<=$len; $i++){
	my $str = join(".",$filename[$i],"tmp");
	`more $filename[$i] | grep $chr."fa" > $str`;
     	push (@cutfile ,$str);
}

`cat @cutfile >combined_tmp.txt`;
`rm -rf *tmp`;

my $infile  = "combined_tmp.txt";
my $infile_uniq  = "combined_uniq.txt";
my $outfile = $chr."_".$filename[0]."_fragL".$L."_bin".$binsize.".txt";

open IN, "$infile" or die "Cannot open $infile\n";
open OUT_uniq, ">$infile_uniq" or die "Cannot open $infile_uniq\n";
open OUT, ">$outfile" or die "Cannot open $outfile\n";

my %seen =();
while(<IN>){
	chomp;	
	my ($t1, $chrt, $str, $pos, $prob) = split /\s+/, $_;
	my $read_length = 28;
	$pos = $pos + $read_length - $L if($str eq "-");
	$str = "+" if($str eq "-");
	my $id = join("",$chrt,$pos);
	$seen{$id}++;
	next if $seen{$id}>$collapse;
	next if $pos<0;
	print OUT_uniq "$t1\t$chrt\t$str\t$pos\t$prob\n";
}

close IN;
close OUT_uniq; 
`rm -rf $infile`;
open IN, "$infile_uniq" or die "Cannot open $infile_uniq\n";

my @bin_count = ();

   while (<IN>) {
	chomp;
	
	my ($t1, $chrt, $str, $pos, $prob) = split /\s+/, $_;
	my $read_length = 28;
	my $L_tmp = $L;  
	if($L < $read_length){$L_tmp = $read_length};
	if ($str eq "+") {
         	my $bin_start = int($pos/$binsize) ;
		my $bin_stop = int(($pos + $L_tmp - 1 )/$binsize) ;
		for(my $i = $bin_start; $i <= $bin_stop; $i++){$bin_count[$i]+=$prob};	
	}
	elsif ($str eq "-") {
		my $start_tmp = $pos + $read_length - $L_tmp;
		$start_tmp = 1 if ($start_tmp < 1);
		my $bin_start = int($start_tmp/$binsize) ;
		my $bin_stop = int(($pos + $read_length - 1)/$binsize) ;
		for(my $i = $bin_start; $i <= $bin_stop; $i++){$bin_count[$i]+=$prob};
	}
	else {
		print "PROBLEM\n";
	}
    }
    
    close IN;
    `rm -rf $infile_uniq`;

    for( my $i = 0; $i< scalar(@bin_count); $i++ ){
	my $coord = $i*$binsize;
	if($bin_count[$i]){print OUT "$chr\t$coord\t$bin_count[$i]\n";}
	else {print OUT "$chr\t$coord\t0\n";}
    }

    close OUT;
     print "Pre-processed $chr using frag length of $L and binsize of $binsize!\n";







